## ------------------------------------------------------------------------
## 1. Möglichkeit: mit pmin und pmax
myfun1 <- function(x, M){
  pmax(-M, pmin(x, M))
}
## Körper der Funktion
body(myfun1)
## Argumentliste der Funktion
args(myfun1)

## ------------------------------------------------------------------------
## Vektor zum Testen der Funktion
x <- seq(-10, 10, length = 20)
## Aufruf mit Argumentnamen
myfun1(x = x, M = 5)
## Nicht nötig, die korrekte Reihenfolge der Argumente zu beachten
myfun1(M = 5, x = x)
## Argumente per Position
myfun1(x, 5)
## Vorsicht, nicht das Gewünschte!
myfun1(5, x)

## ------------------------------------------------------------------------
## 2. Möglichkeit: mit Default-Belegung von M
myfun2 <- function(x, M = 5){
  pmax(-M, pmin(x, M))
}
## Damit kann beim Aufruf auf die Angabe von M verzichtet werden
myfun2(x)

## ------------------------------------------------------------------------
## 3. Möglichkeit: mit Indikatoren
myfun3 <- function(x, M = 5){
  ## Ist |x| > M?
  ind <- abs(x) > M
  ## Modifiziere alle Einträge von x mit |x| > M
  x[ind] <- sign(x[ind])*M
  x
}
myfun3(x)

## ------------------------------------------------------------------------
## 4. Möglichkeit: mit zusätzlicher Fehlerkontrolle
myfun4 <- function(x, M = 5){
  ## Fehler, falls x fehlt
  if(missing(x))
    stop("Sie müssen x angeben!")
  ## Fehler, falls x oder M keine Zahlen beinhalten
  if(!is.numeric(x) | !is.numeric(M))
    stop("x und M müssen numerische Variablen sein!")
  ## Warnung, falls M mehr als einen Eintrag besitzt
  if(length(M) > 1){
    warning("Die Länge von M ist größer 1.
            Es wird nur das erste Element von M verwendet.")
    M <- M[1]
  }
  ## Fehler, falls M <= 0 ist
  stopifnot(M > 0)
  ## Statusmeldung
  message("Die Berechnung wird jetzt durchgeführt.")

  pmax(-M, pmin(x, M))
}
myfun4() ## Fehler!
myfun4(x, 'test') ## Fehler!
myfun4(x, M = -1) ## Fehler!
myfun4(x, M = c(5, 6)) ## Warnung!
myfun4(x)

## ------------------------------------------------------------------------
myfun5 <- function(x, M, g, ...){
  pmax(-M, pmin(g(x, ...), M))
}
## zum Beispiel
x <- seq(-1.9, -1.1, by = 0.1)
## Stutze 2. Ableitung der psigamma Funktion
myfun5(x, g = psigamma, M = 100, deriv = 2)

## ------------------------------------------------------------------------
myfun6 <- function(x, Funktion, ..., Schranke){
  pmax(-Schranke, pmin(Funktion(x, ...), Schranke))
}
## korrekter Aufruf
myfun6(x, psigamma, 2, Schranke = 100)
## korrekter Aufruf
myfun6(x = x, F = psigamma, deriv = 2, Schranke = 100)
## korrekter Aufruf
myfun6(x = x, F = psigamma, Schranke = 100, deriv = 2)
## Argumente nach "..." müssen vollständig angegeben werden
myfun6(x = x, Fun = psigamma, deriv = 2, S = 100) ## Fehler!
## Argument, die zu "..." gehören, können abgekürzt werden
myfun6(x = x, Fun = psigamma, der = 2, Schranke = 100)
## am besten und klarsten
myfun6(x = x, Funktion = psigamma, deriv = 2, Schranke = 100)

## ------------------------------------------------------------------------
myfun7 <- function(x, M, g, ...){
  ## Abfangen des Funktionsaufrufs
  cl <- match.call(expand.dots = FALSE)
  ## Abfangen der Argumente in ...
  args <- list(...)
  if(length(grep("psigamma", cl)) > 0){
    if(hasArg(deriv))
      cat("Auf", M, "gestutzte Version der", args[["deriv"]], "-ten Ableitung",
          "der psigamma-Funktion\n")
    else
      cat("Auf", M, "gestutzte Version der psigamma-Funktion\n")
  }
  pmax(-M, pmin(g(x, ...), M))
}
myfun7(x, M = 5, g = psigamma)
myfun7(x, M = 250, g = psigamma, deriv = 2)

## ------------------------------------------------------------------------
## old.op <- options(error = recover)
## myfun6(x = y, Fun = psigamma, deriv = 2, S = 100)
## options(old.op)

## ------------------------------------------------------------------------
## old.op <- options(warn = 2)
## myfun4(x, M = c(5, 6))
## options(old.op)

## ------------------------------------------------------------------------
## plot(1:10, 1:8)
## traceback()

## ------------------------------------------------------------------------
## old.op <- options(error = dump.frames)
## myfun6(x = y, Fun = log, base = 2, S = 1)
## debugger(dump = last.dump)
## options(old.op)

## ------------------------------------------------------------------------
fibonacci <- function(n){
  if(n == 0) return(0)
  if(n == 1) return(c(0, 1))
  ## Verwendung einer for-Schleife
  an <- numeric(n+1)
  an[2] <- 1
  for(i in 3:(n+1)){
    an[i] <- an[i-2] + an[i-1]
    print(an) ## "Debugging"
  }
  an
}
fibonacci(5)

## ------------------------------------------------------------------------
fibonacci <- function(n, method = 'Schleife'){
  if(n == 0) return(0)
  if(n == 1) return(c(0, 1))
  ## Verwendung einer for-Schleife
  an <- numeric(n+1)
  an[2] <- 1
  for(i in 3:(n+1)){
    cat('Iteration:\t', i, '\n') ## "Debugging"
    an[i] <- an[i-2] + an[i-1]
    cat('Ergebnis der Berechnungen:\t', an, '\n') ## "Debugging"
  }
  an
}
fibonacci(5)

## ------------------------------------------------------------------------
## ## Die Funktion ohne Debugging-Einträge
## fibonacci <- function(n){
##   if(n == 0) return(0)
##   if(n == 1) return(c(0, 1))
##   ## Verwendung einer for-Schleife
##   an <- numeric(n+1)
##   an[2] <- 1
##   for(i in 3:(n+1)){
##     an[i] <- an[i-2] + an[i-1]
##   }
##   return(an)
## }
## debug("fibonacci")
## fibonacci(5)
## undebug('fibonacci')

## ------------------------------------------------------------------------
## fibonacci <- function(n){
##   if(n == 0) return(0)
##   if(n == 1) return(c(0, 1))
##   ## Verwendung einer for-Schleife
##   an <- numeric(n+1)
##   an[2] <- 1
##   for(i in 3:(n+1)){
##     an[i] <- an[i-2] + an[i-1]
##     browser()
##   }
##   return(an)
## }
## fibonacci(5)

## ------------------------------------------------------------------------
## Die Funktion ohne Debugging
fibonacci <- function(n){
  if(n == 0) return(0)
  if(n == 1) return(c(0, 1))
  ## Verwendung einer for-Schleife
  an <- numeric(n+1)
  an[2] <- 1
  for(i in 3:(n+1)){
    an[i] <- an[i-2] + an[i-1]
  }
  return(an)
}
## Körper der Funktion
as.list(body(fibonacci))

## ------------------------------------------------------------------------
## trace(what = 'fibonacci', tracer = browser, at = 6)
## body(fibonacci)
## fibonacci(5)

## ------------------------------------------------------------------------
trace(what = 'fibonacci', tracer = browser, exit = browser)
body(fibonacci)

## ------------------------------------------------------------------------
untrace("fibonacci")

## ------------------------------------------------------------------------
## ## zwar Fehlermeldung aber kein Abbruch
## res <- try(plot(1:10, 1:8))
## ## oder ohne Ausgabe der Fehlermeldung
## res <- try(plot(1:10, 1:8), silent = TRUE)

