## ------------------------------------------------------------------------
## Wir betrachten x = -2
x <- -2
x
## Berechnung von y = |x|
if(x >= 0){
    y <- x
}else{
    y <- -x
}
y
## oder kürzer, aber weniger übersichtlich
if(x >= 0) y <- x else y <- -x
y

## ------------------------------------------------------------------------
x <- c(2, -2)
x
if(x >= 0) y <- x else y <- -x
## es wird nur das erste Element des logischen Vektors (x >= 0) berücksichtigt!
## Vergleiche
x >= 0
## somit
y

## ------------------------------------------------------------------------
x
y <- ifelse(x >= 0, x, -x)
y

## ------------------------------------------------------------------------
y <- x
## Indikatorvariable
ind <- x < 0
y[ind] <- -x[ind]
x
y

## ------------------------------------------------------------------------
x <- c(2, NA, -2)
x
## NAs in der Bedingung führen zu NAs im Ergebnis
ifelse(x >= 0, x, -x)
y <- x
ind <- x < 0
ind
## daher
-x[ind]
## möglicher Ausweg: entferne NA vor der Berechnung
y <- x[!is.na(x)]
ind <- y < 0
y[ind] <- -y[ind]
y
x

## ------------------------------------------------------------------------
abs(x)

## ------------------------------------------------------------------------
## 1. Möglichkeit: Verwendung mehrerer if-Anweisungen
lage1 <- function(x){
  ## Falls factor -> gib Modus zurück
  if(is.factor(x)){ return(names(which.max(table(x)))) }
  ## Falls integer -> gib Median zurück
  if(is.integer(x)){ return(median(x)) }
  ## Falls double -> gib arithmetisches Mittel zurück
  if(is.double(x)){ return(mean(x)) }
  ## Falls nichts davon erfüllt ist, wird der Vektor selbst zurückgegeben.
  x
}
x <- factor(rpois(10, lambda = 3))
## Absolute Häufigkeiten
table(x)
## Modus
lage1(x)
y <- seq_len(10)
y
## Median
lage1(y)
z <- rnorm(10)
z
## Arithmetisches Mittel
lage1(z)
## Nicht vorgesehener Datentyp
lage1(letters[1:5])
###########################################################
## 2. Möglichkeit: switch
lage2 <- function(x){
  ## Explizite Behandlung von factor als S3-Klasse
  if(is.factor(x)){
    method <- "factor"
  }else{
    method <- typeof(x)
  }
  res <- switch(method,
                "factor" = names(which.max(table(x))),
                "integer" = median(x),
                "double" = mean(x),
                ## im Fall, dass keines davon zutrifft
                x)
  res
}
## Modus
lage2(x)
## Median
lage2(y)
## Arithmetisches Mittel
lage2(z)
##  Nicht vorgesehener Datentyp
lage2(letters[1:5])

## ------------------------------------------------------------------------
lage3 <- function(x){
  if(is.factor(x)){
    method <- 1
  }else{
    method <- typeof(x)
  }
  res <- switch(method,
                "factor" = names(which.max(table(x))),
                "integer" = median(x),
                "double" = mean(x),
                ## im Fall, dass keines davon zutrifft
                x)
  res
}
## Modus
lage3(x)
## Median
lage3(y)
## Arithmetisches Mittel
lage3(z)
##  Nicht vorgesehener Datentyp
lage3(letters[1:5])

## ------------------------------------------------------------------------
## Wir legen die Matrix an
A <- matrix(rnorm(50), nrow = 10)
A
## Initialisierung der Ergebnisvariable
summe <- numeric(nrow(A))
## for-Schleife: i durchläuft die Werte 1 bis nrow(A)
for(i in 1:nrow(A)){
  summe[i] <- sum(A[i,])
}
summe
## Zur Kontrolle
rowSums(A)

## ------------------------------------------------------------------------
1:0

## ------------------------------------------------------------------------
seq_len(0)
## Initialisierung der Ergebnisvariable
summe <- numeric(nrow(A))
## for-Schleife: i durchläuft die Werte 1 bis nrow(A)
for(i in seq_len(nrow(A))){
  summe[i] <- sum(A[i,])
}
summe

## ------------------------------------------------------------------------
B <- A
rownames(B) <- paste('Zeile', 1:10)
B
## Initialisierung der Ergebnisvariable
summe.B <- numeric(nrow(B))
## Einführen eines names Attributs für die Indizierung
names(summe.B) <- rownames(B)
## Name durchläuft die Zeilennamen von B
for(Name in rownames(B)){
  summe.B[Name] <- sum(B[Name,])
}
summe.B
## Zur Kontrolle
rowSums(B)

## ------------------------------------------------------------------------
## Initialisierung der Ergebnisvariable
summe <- numeric(nrow(A))
## Initialisierung einer Variable zur Speicherung der aktuellen Iteration
iter <- 0
## Durchlauf die Schleife solange die Iterationszahl kleiner als die
## Anzahl der Zeilen von A ist.
while(iter < nrow(A)){
  iter <- iter+1
  summe[iter] <- sum(A[iter,])
}
summe
## Zur Kontrolle
rowSums(A)

## ------------------------------------------------------------------------
## Initialisierung der Ergebnisvariable
summe <- numeric(nrow(A))
## Initialisierung einer Variable zur Speicherung der aktuellen Iteration
iter <- 1
## Durchlauf der Schleife solange iter <= Zeilenzahl von A
repeat{
  summe[iter] <- sum(A[iter,])
  iter <- iter + 1
  if(iter > nrow(A)) break
}
summe
## Zur Kontrolle
rowSums(A)

## ------------------------------------------------------------------------
summe <- apply(A, 1, sum)
summe

## ------------------------------------------------------------------------
## repeat{
##   R-code ...
##   if(Bedingung) next
##   R-code ...
##   if(Abbruchbedingung) break
## }

## ------------------------------------------------------------------------
library(microbenchmark)
library(ggplot2)
A <- matrix(rnorm(2500), nrow = 500)
res <- microbenchmark(
  {
    summe <- numeric(nrow(A))
    for(i in seq_len(nrow(A))){
      summe[i] <- sum(A[i,])
    }
  },
  {
    summe <- numeric(nrow(A))
    iter <- 0
    while(iter < nrow(A)){
    iter <- iter+1
    summe[iter] <- sum(A[iter,])
    }
  },
  {
    summe <- numeric(nrow(A))
    iter <- 1
    repeat{
      summe[iter] <- sum(A[iter,])
      iter <- iter + 1
      if(iter > nrow(A)) break
    }
  },
  apply(A, 1, sum), rowSums(A), times = 10000, unit = "relative")
levels(res$expr) <- c("for-Schleife", "while-Schleife", "repeat-Schleife",
                      "apply", "rowSums")
## relative Unterschiede bezogen auf das schnellste Verfahren
summary(res)[,1:7]
autoplot(res)

