## ------------------------------------------------------------------------
x <- factor(1:10)
class(x)
## oder auch
data.class(x)
## dies entspricht
attr(x, "class")
## alle Attribute
attributes(x)
## Speichermodus/Typ nicht notwendig identisch zur Klasse!
mode(x)
## jedoch: factor ist nicht abgeleitet von vector oder numeric!
is.numeric(x)
is.vector(x)

## ------------------------------------------------------------------------
y <- ordered(1:10)
## Vererbung: factor <- ordered
class(y)

## ------------------------------------------------------------------------
is.factor(y)
is.ordered(x)
## oder auch
inherits(x, what = c("ordered", "factor"), which = TRUE)

## ------------------------------------------------------------------------
y
class(y) <- "factor"
y
class(y)

## ------------------------------------------------------------------------
x
class(x) <- "data.frame"
x

## ------------------------------------------------------------------------
x <- factor(c("a", "b", "c"))
y <- as.character(x)
y
class(y)
y <- as.integer(x)
y
class(y)
y <- as.data.frame(x)
y
class(y)

## ------------------------------------------------------------------------
y <- unclass(x) ## x bleibt unverändert
y

## ------------------------------------------------------------------------
class(x) <- NULL
x

## ------------------------------------------------------------------------
D <- data.frame(a = rnorm(10), b = character(10), c = factor(1:10))
class(D)
data.class(D)
attributes(D)
## casting in "matrix"
M <- as.matrix(D)
M
mode(M) ## character-Matrix
## data.matrix erzwingt die Umwandlung in eine numerische Matrix
M <- data.matrix(D)
M
mode(M) ## numeric-Matrix

## ------------------------------------------------------------------------
## Erzeugen eines Datensatzes
x <- sample(1:10, 100, replace = TRUE)
y <- x + rnorm(100)
## Scatterplot der Daten
plot(x, y)
## Berechnung eines einfachen linearen Modells
lm1 <- lm(y ~ x)
class(lm1) ## Klasse "lm"
data.class(lm1)
attributes(lm1)
mode(lm1)

## ------------------------------------------------------------------------
is.list(lm1)

## ------------------------------------------------------------------------
z <- structure(data.frame(a = 10, b = 5), class = "MeineNeueS3Klasse")
z ## es gibt noch keine eigene print-Methode für diese neue Klasse
class(z)
data.class(z)

## ------------------------------------------------------------------------
z <- structure(data.frame(a = 10, b = 5),
               class = c("MeineNeueS3Klasse", "data.frame"))
class(z)
z ## Aufruf der print-Methode für data.frame

## ------------------------------------------------------------------------
x <- numeric(10) ## identisch zu x <- double(10) bzw. x <- real(10)
class(x)
data.class(x)
## aber kein Klassenattribut
attributes(x)
## In diesem Spezialfall entspricht Aufruf von "class" dem Aufruf von "mode"
mode(x)

## ------------------------------------------------------------------------
## vector <- numeric
is.numeric(x)
is.vector(x)

## ------------------------------------------------------------------------
## vector <- raw
x <- raw(10)
class(x)
data.class(x)
attributes(x)
mode(x)
is.raw(x)
is.vector(x)

## ------------------------------------------------------------------------
## vector <- list
x <- vector("list", 3)
x
class(x)
data.class(x)
attributes(x)
mode(x)
is.vector(x)
is.list(x)
## möglicherweise Attribut names
x <- list(a = 1, b = 2)
class(x)
attributes(x)
## casting: Umwandeln in ein Objekt einer anderen Klasse
## hier: von "list" zu "vector"
y <- unlist(x)
is.list(y)
is.vector(y)

## ------------------------------------------------------------------------
## vector <- numeric <- integer
x <- integer(10)
class(x) ## spezielle implizite Klasse!
## Achtung!
data.class(x)
mode(x) ## mode nicht identisch zu class!
is.vector(x)
is.numeric(x)
is.integer(x)

## ------------------------------------------------------------------------
A <- array(rnorm(12), dim = c(2, 2, 3))
class(A)
data.class(A)
attributes(A)
mode(A)

## ------------------------------------------------------------------------
methods(class = "vector")
methods(class = "numeric")
methods(class = "lm")

## ------------------------------------------------------------------------
methods("summary")

## ------------------------------------------------------------------------
## 1. Möglichkeit
summary.matrix
## 2. Möglichkeit
getS3method(f = "summary", class = "matrix")

## ------------------------------------------------------------------------
## 1. Möglichkeit
getS3method(f = "summary", class = "loess")

## ------------------------------------------------------------------------
## 2. Möglichkeit
stats:::summary.loess

## ------------------------------------------------------------------------
## 3. Möglichkeit
getAnywhere("summary.loess")

## ------------------------------------------------------------------------
z <- structure(data.frame(a = 10, b = 5), class = "MeineNeueS3Klasse")

## ------------------------------------------------------------------------
z
## bzw.
print(z)
## bzw.
print.default(z)

## ------------------------------------------------------------------------
print.data.frame(z)

## ------------------------------------------------------------------------
z <- structure(data.frame(a = 10, b = 5),
               class = c("MeineNeueS3Klasse", "data.frame"))
z
## ist jetzt identisch zu
print.data.frame(z)

## ------------------------------------------------------------------------
estimate <- function(x, estimator, ...){
  class(x) <- data.class(x)
  UseMethod("estimate", x) ## oder auch nur UseMethod("estimate")
}

## ------------------------------------------------------------------------
estimate.default <- function(x, estimator, ...){
  structure(list(sample = x, estimate = estimator(x, ...)),
            class = "estimate")
}

## ------------------------------------------------------------------------
x <- rnorm(10)
estimate(x, mean)
estimate(x, median)
M <- matrix(rnorm(20), ncol = 2)
## vermutlich (noch) nicht das gewünschte Ergebnis
estimate(M, mean)
## vermutlich (noch) nicht das gewünschte Ergebnis
estimate(M, median)

## ------------------------------------------------------------------------
print.estimate <- function(x, ...){
  cat("Sample:\n")
  cat(str(x$sample))
  cat("\nEstimate:\n")
  print(x$estimate)
  ## Rückgabewert der Funktion wird nicht ausgegeben
  invisible(x)
}
## o.k.
estimate(x, mean)
## vermutlich (noch) nicht das gewünschte Ergebnis
estimate(M, mean)

## ------------------------------------------------------------------------
estimate.matrix <- function(x, estimator, ...){
  estimate <- apply(x, 2, estimator, ...)
  structure(list(sample = x, estimate = estimate), class = "estimate")
}
## Tests
estimate(M, mean)
estimate(M, median)
## jedoch
estimate(as.data.frame(M), mean)

## ------------------------------------------------------------------------
estimate.data.frame <- function(x, estimator, ...){
    ## Casting von data.frame nach matrix
    y <- data.matrix(x)
    ## Rufe Methode für matrix auf
    UseMethod("estimate", y)
}
## Tests
estimate(as.data.frame(M), mean)
estimate(as.data.frame(M), median)
## Zum Vergleich
estimate(M, mean)
estimate(M, median)

## ------------------------------------------------------------------------
estimate.numeric <- function(x, estimator, ...){
  ## Abfangen von integer-Vektoren
  if(is.integer(x)){
    warning("x ist ein integer Vektor - prüfen Sie, ob estimator anwendbar ist!")
  }
  NextMethod("estimate", x, ...)
}
## Tests
estimate(x, mean)

x <- as.integer(rpois(10, lambda = 3))
estimate(x, mean)
## x könnte evtl. ein ordinales Merkmal sein
estimate(x, median)

## ------------------------------------------------------------------------
estimate <- function(estimator, x, ...){
    class(x) <- data.class(x)
    UseMethod("estimate", x)
}

